@@ -7,6 +7,7 @@ import (
 	"net"
 	"strings"
 
+	"github.com/lima-vm/lima/pkg/limayaml"
 	"github.com/miekg/dns"
 	"github.com/sirupsen/logrus"
 )
@@ -19,6 +20,8 @@ type Handler struct {
 	clientConfig *dns.ClientConfig
 	clients      []*dns.Client
 	IPv6         bool
+	cname        map[string]string
+	ip           map[string]net.IP
 }
 
 type Server struct {
@@ -44,7 +47,7 @@ func newStaticClientConfig(ips []net.IP) (*dns.ClientConfig, error) {
 	return dns.ClientConfigFromReader(r)
 }
 
-func newHandler(IPv6 bool) (dns.Handler, error) {
+func newHandler(IPv6 bool, hosts map[string]string) (dns.Handler, error) {
 	cc, err := dns.ClientConfigFromFile("/etc/resolv.conf")
 	if err != nil {
 		fallbackIPs := []net.IP{net.ParseIP("8.8.8.8"), net.ParseIP("1.1.1.1")}
@@ -62,6 +65,15 @@ func newHandler(IPv6 bool) (dns.Handler, error) {
 		clientConfig: cc,
 		clients:      clients,
 		IPv6:         IPv6,
+		cname:        make(map[string]string),
+		ip:           make(map[string]net.IP),
+	}
+	for host, address := range hosts {
+		if ip := net.ParseIP(address); ip != nil {
+			h.ip[host] = ip
+		} else {
+			h.cname[host] = limayaml.Cname(address)
+		}
 	}
 	return h, nil
 }
@@ -79,17 +91,43 @@ func (h *Handler) handleQuery(w dns.ResponseWriter, req *dns.Msg) {
 			Class:  q.Qclass,
 			Ttl:    5,
 		}
-		switch q.Qtype {
+		qtype := q.Qtype
+		switch qtype {
 		case dns.TypeAAAA:
 			if !h.IPv6 {
-				break
+				// A "correct" answer would be to set `handled = true` and return a NODATA response.
+				// Unfortunately some older resolvers use a slow random source to set the transaction id.
+				// This creates a problem on M1 computers, which are too fast for that implementation:
+				// Both the A and AAAA queries might end up with the same id. Returning NODATA for AAAA
+				// is faster, so would arrive first, and be treated as the response to the A query.
+				// To avoid this, we will treat an AAAA query as an A query when IPv6 has been disabled.
+				// This way it is either a valid response for an A query, or the A records will be discarded
+				// by a genuine AAAA query, resulting in the desired NODATA response.
+				qtype = dns.TypeA
 			}
 			fallthrough
 		case dns.TypeCNAME, dns.TypeA:
-			cname, err := net.LookupCNAME(q.Name)
-			if err != nil {
+			cname := q.Name
+			seen := make(map[string]bool)
+			for {
+				// break cyclic definition
+				if seen[cname] {
+					break
+				}
+				if _, ok := h.cname[cname]; ok {
+					seen[cname] = true
+					cname = h.cname[cname]
+					continue
+				}
 				break
 			}
+			var err error
+			if _, ok := h.ip[cname]; !ok {
+				cname, err = net.LookupCNAME(cname)
+				if err != nil {
+					break
+				}
+			}
 			if cname != "" && cname != q.Name {
 				hdr.Rrtype = dns.TypeCNAME
 				a := &dns.CNAME{
@@ -99,22 +137,28 @@ func (h *Handler) handleQuery(w dns.ResponseWriter, req *dns.Msg) {
 				reply.Answer = append(reply.Answer, a)
 				handled = true
 			}
-			if q.Qtype == dns.TypeCNAME {
+			if qtype == dns.TypeCNAME {
 				break
 			}
 			hdr.Name = cname
-			addrs, err := net.LookupIP(q.Name)
+			var addrs []net.IP
+			if _, ok := h.ip[cname]; ok {
+				addrs = []net.IP{h.ip[cname]}
+				err = nil
+			} else {
+				addrs, err = net.LookupIP(cname)
+			}
 			if err == nil && len(addrs) > 0 {
 				for _, ip := range addrs {
 					var a dns.RR
 					ipv6 := ip.To4() == nil
-					if q.Qtype == dns.TypeA && !ipv6 {
+					if qtype == dns.TypeA && !ipv6 {
 						hdr.Rrtype = dns.TypeA
 						a = &dns.A{
 							Hdr: hdr,
 							A:   ip.To4(),
 						}
-					} else if q.Qtype == dns.TypeAAAA && ipv6 {
+					} else if qtype == dns.TypeAAAA && ipv6 {
 						hdr.Rrtype = dns.TypeAAAA
 						a = &dns.AAAA{
 							Hdr:  hdr,
@@ -130,12 +174,17 @@ func (h *Handler) handleQuery(w dns.ResponseWriter, req *dns.Msg) {
 		case dns.TypeTXT:
 			txt, err := net.LookupTXT(q.Name)
 			if err == nil && len(txt) > 0 {
-				a := &dns.TXT{
-					Hdr: hdr,
-					Txt: txt,
+				for _, s := range txt {
+					a := &dns.TXT{
+						Hdr: hdr,
+					}
+					// Per RFC7208 3.3, when a TXT answer has multiple strings, the answer must be treated as
+					// a single concatenated string. net.LookupTXT is pre-concatenating such answers, which
+					// means we need to break it back up for this resolver to return a valid response.
+					a.Txt = chunkify(s, 255)
+					reply.Answer = append(reply.Answer, a)
+					handled = true
 				}
-				reply.Answer = append(reply.Answer, a)
-				handled = true
 			}
 		case dns.TypeNS:
 			ns, err := net.LookupNS(q.Name)
@@ -219,8 +268,8 @@ func (h *Handler) ServeDNS(w dns.ResponseWriter, req *dns.Msg) {
 	}
 }
 
-func Start(udpLocalPort, tcpLocalPort int, IPv6 bool) (*Server, error) {
-	h, err := newHandler(IPv6)
+func Start(udpLocalPort, tcpLocalPort int, IPv6 bool, hosts map[string]string) (*Server, error) {
+	h, err := newHandler(IPv6, hosts)
 	if err != nil {
 		return nil, err
 	}
@@ -247,3 +296,15 @@ func Start(udpLocalPort, tcpLocalPort int, IPv6 bool) (*Server, error) {
 	}
 	return server, nil
 }
+
+func chunkify(buffer string, limit int) []string {
+	var result []string
+	for len(buffer) > 0 {
+		if len(buffer) < limit {
+			limit = len(buffer)
+		}
+		result = append(result, buffer[:limit])
+		buffer = buffer[limit:]
+	}
+	return result
+}
