@@ -10,5 +10,5 @@ import (
 )
 
 func forwardTCP(ctx context.Context, sshConfig *ssh.SSHConfig, port int, local, remote string, verb string) error {
-	return forwardSSH(ctx, sshConfig, port, local, remote, verb)
+	return forwardSSH(ctx, sshConfig, port, local, remote, verb, false)
 }
