@@ -51,6 +51,7 @@ func TestFillDefault(t *testing.T) {
 		CPUType: map[Arch]string{
 			AARCH64: "cortex-a72",
 			X8664:   "qemu64",
+			RISCV64: "rv64",
 		},
 		CPUs:   pointer.Int(4),
 		Memory: pointer.String("4GiB"),
@@ -88,6 +89,7 @@ func TestFillDefault(t *testing.T) {
 		HostIP:         api.IPv4loopback1,
 		HostPortRange:  [2]int{1, 65535},
 		Proto:          TCP,
+		Reverse:        false,
 	}
 
 	// ------------------------------------------------------------------------------------
@@ -211,6 +213,7 @@ func TestFillDefault(t *testing.T) {
 		CPUType: map[Arch]string{
 			AARCH64: "arm64",
 			X8664:   "amd64",
+			RISCV64: "riscv64",
 		},
 		CPUs:   pointer.Int(7),
 		Memory: pointer.String("5GiB"),
@@ -351,6 +354,7 @@ func TestFillDefault(t *testing.T) {
 		CPUType: map[Arch]string{
 			AARCH64: "uber-arm",
 			X8664:   "pentium",
+			RISCV64: "sifive-u54",
 		},
 		CPUs:   pointer.Int(12),
 		Memory: pointer.String("7GiB"),
